/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the examples of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial Usage
** Licensees holding valid Qt Commercial licenses may use this file in
** accordance with the Qt Commercial License Agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Nokia.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
** $QT_END_LICENSE$
**
****************************************************************************/


#include <QtWidgets>
#include <QtNetwork>

#include "receiver.h"

Receiver::Receiver(char * destIp,unsigned int portIn, unsigned int portOut,int posx,int posy,int sizex,int sizey,QWidget *parent)
    : QDialog(parent)
{
    m_destIP=strdup(destIp);

    m_portIn=portIn;
    m_portOut=portOut;

    statusLabel = new QLabel(tr("En attente de datagrammes UDP depuis le PIC32"));

    commutTimerButton=new QPushButton(tr("start clignote LED0"));

    quitButton = new QPushButton(tr("&Quit"));
   // quitButton = new QPushButton(Qchaine);
    led0Button = new QPushButton(tr("Led&0"));
    led1Button = new QPushButton(tr("Led&1"));
    led2Button = new QPushButton(tr("Led&2"));

#ifdef SLIDERACTIF   //slider
    sliderLabel = new QLabel(tr("affichage de la variable compteur:"));

    //http://doc.trolltech.com/4.2/widgets-sliders.html
    slider = new QSlider(Qt::Horizontal);
    slider->setFocusPolicy(Qt::StrongFocus);
    slider->setTickPosition(QSlider::TicksBothSides);
    slider->setTickInterval(100);
    slider->setMaximum(1000);
    slider->setMinimum(0);
    slider->setSingleStep(1);
#endif



    //! [0]
    udpSocket = new QUdpSocket(this);
    udpSocket->bind(portIn, QUdpSocket::ShareAddress);
    //bool QUdpSocket::bind ( quint16 port, BindMode mode )
    //QUdpSocket::ShareAddress	0x1	Allow other services to bind to the same address and port. This is useful when multiple processes share the load of a single service by listening to the same address and port (e.g., a web server with several pre-forked listeners can greatly improve response time). However, because any service is allowed to rebind, this option is subject to certain security considerations. Note that by combining this option with ReuseAddressHint, you will also allow your service to rebind an existing shared address. On Unix, this is equivalent to the SO_REUSEADDR socket option. On Windows, this option is ignored.

    //! [0]

    //! [1]
    connect(udpSocket, SIGNAL(readyRead()),
            this, SLOT(processPendingDatagrams()));
    //! [1]
    connect(quitButton, SIGNAL(clicked()), this, SLOT(close()));
    //   connect(led0Button, SIGNAL(clicked()), this, SLOT(close()));

    connect(commutTimerButton, SIGNAL(clicked()), this, SLOT(commutTimer()));


    connect(led0Button, SIGNAL(clicked()), this, SLOT(sendUDP0()));
    connect(led1Button, SIGNAL(clicked()), this, SLOT(sendUDP1()));
    connect(led2Button, SIGNAL(clicked()), this, SLOT(sendUDP2()));

    QHBoxLayout *buttonLayout = new QHBoxLayout;
    buttonLayout->addStretch(1);
    buttonLayout->addWidget(quitButton);
    buttonLayout->addWidget(commutTimerButton);
    buttonLayout->addStretch(1);
#ifdef SLIDERACTIF
    buttonLayout->addWidget(sliderLabel);
    buttonLayout->addWidget(slider);
    buttonLayout->addStretch(1);
#endif
    QHBoxLayout *button1Layout = new QHBoxLayout;
    button1Layout->addStretch(1);
    button1Layout->addWidget(led0Button);
    button1Layout->addStretch(1);
    button1Layout->addWidget(led1Button);
    button1Layout->addStretch(1);
    button1Layout->addWidget(led2Button);
    button1Layout->addStretch(1);


    sw1 = new QCheckBox("sw 1", this);
    sw2 = new QCheckBox("sw 2", this);
    sw3 = new QCheckBox("sw 3", this);

    QHBoxLayout *button2Layout = new QHBoxLayout;
    button2Layout->addStretch(1);
    button2Layout->addWidget(sw1);
    button2Layout->addStretch(1);
    button2Layout->addWidget(sw2);
    button2Layout->addStretch(1);
    button2Layout->addWidget(sw3);
    button2Layout->addStretch(1);

    QVBoxLayout *mainLayout = new QVBoxLayout;
    mainLayout->addLayout(buttonLayout);
    mainLayout->addLayout(button1Layout);
    mainLayout->addLayout(button2Layout);
    mainLayout->addWidget(statusLabel);
    setLayout(mainLayout);

   // setWindowTitle(tr("Communication UDP avec PIC32 par BVDP 2011 Fait clignoter la led 0 à 0.25Hz"));
    char chaine[1000];
   // sprintf(chaine,"@BVDP2011: Reception sur port %d\n Emission sur port %d vers IP: %s",m_portIn,m_portOut,m_destIP);
 sprintf(chaine,"@BVDP2011:Port Rec %d Port Em %d vers IP: %s",m_portIn,m_portOut,m_destIP);
    QString Qchaine(chaine);

    setWindowTitle(tr(chaine));
    Timer=new QTimer(this);
        Timer->setInterval(2000);
        connect(Timer, SIGNAL(timeout()), this, SLOT(on_timer_Event()));
          //Timer->start();
//this->setGeometry(100,100,800,200);
this->setGeometry(posx,posy,sizex,sizey);

}


void Receiver::sendUDP0()
{
    sendUDP(0);
}
void Receiver::sendUDP1()
{
    sendUDP(1);
}
void Receiver::sendUDP2()
{
    sendUDP(2);
}


void Receiver::commutTimer()
{
if (Timer->isActive())
    {
    Timer->stop();
    commutTimerButton->setText("start clignote LED0");
    }
else
    {
         Timer->start();
         commutTimerButton->setText("stop clignote LED0");

     }
}

void Receiver::on_timer_Event()
{
 sendUDP(0);
}


void Receiver::sendUDP(int i)
{

 /*   //QHostAddress DestIP("192.168.1.11");
    QHostAddress DestIP("169.254.163.229");
    quint16 DestPort=27016;
    QUdpSocket udpEmiSocket;
    char chaine[]="  test\n";
    // for (int i=0;i<10;i++)
    sprintf(chaine,"%d",i) ;
    udpEmiSocket.writeDatagram((const char *) chaine,(qint64)strlen(chaine)+1,DestIP, DestPort);
*/
   //   QHostAddress DestIP("169.254.163.229");

    //    QHostAddress DestIP("192.168.1.101");
//    QHostAddress DestIP("192.168.1.100");
    QHostAddress DestIP(m_destIP);



//    QHostAddress DestIP("192.168.1.4");


    quint16 DestPort=m_portOut;
      char chaine[]="  test\n";
     // for (int i=0;i<10;i++)
     sprintf(chaine,"%d",i) ;
     udpSocket->writeDatagram((const char *) chaine,(qint64)strlen(chaine)+1,DestIP, DestPort);


}

void Receiver::processPendingDatagrams()
{
        int cpt,b1,b2,b3;
    //! [2]
    while (udpSocket->hasPendingDatagrams()) {
        QByteArray datagram;
        datagram.resize(udpSocket->pendingDatagramSize());
        udpSocket->readDatagram(datagram.data(), datagram.size());

//datagram.truncate(10); //limite à 10 cars
        statusLabel->setText(tr("Received datagram: \"%1\"")
                             .arg(datagram.data()));

        sscanf(datagram.data(),"compteur= %d b1:%d b2:%d b3:%d\n",&cpt,&b1,&b2,&b3);

        this->sw1->setChecked((b1==1));
        this->sw2->setChecked((b2==1));
        this->sw3->setChecked((b3==1));
#ifdef SLIDERACTIF
        this->slider->setValue(cpt%1000);
#endif

    }
    //! [2]
}



